==> mx2g_1.1.1.tar.gz: mx-1.1.1/fma/libfma/lf_topo_map.h <==
#ifndef _lf_topo_map_h_
#define _lf_topo_map_h_

/*
 * This is a purely topological map of the network in a form
 * suitable for sending over the network
 */

/*
 * The topo map is represented in memory as one "struct lf_topo_map"
 * follow by "num_xbars" "struct lf_topo_xbar"s, etc.  The following
 * macros are for getting to each array
 */
#define LF_TOPO_XBAR_ARRAY(TM)						\
  ((struct lf_topo_xbar *)((void *)(TM) + sizeof(struct lf_topo_map)))
#define LF_TOPO_NIC_ARRAY(TM)						\
  ((struct lf_topo_nic *)((void *)LF_TOPO_XBAR_ARRAY(TM)		\
   + (sizeof(struct lf_topo_xbar) *					\
     ntohl(((struct lf_topo_map *)(TM))->num_xbars_32))))
#define LF_TOPO_LINK_ARRAY(TM)						\
  ((struct lf_topo_link *)((void *)LF_TOPO_NIC_ARRAY(TM) 		\
   + (sizeof(struct lf_topo_nic) * 					\
     ntohl(((struct lf_topo_map *)(TM))->num_nics_32))))
#define LF_TOPO_LINK_STATE_ARRAY(TM)					\
  ((lf_topo_link_state_t *)((void *)LF_TOPO_LINK_ARRAY(TM)		\
   + (sizeof(struct lf_topo_link) * 					\
     ntohl(((struct lf_topo_map *)(TM))->num_links_32))))

#define LF_TOPO_MAP_SIZE(XBARS, NICS, LINKS)				\
   (sizeof(struct lf_topo_map)						\
   + (sizeof(struct lf_topo_xbar) * (XBARS))				\
   + (sizeof(struct lf_topo_nic) * (NICS))				\
   + (sizeof(struct lf_topo_link) * (LINKS))				\
   + (sizeof(lf_topo_link_state_t) * (LINKS)))
#define LF_TOPO_LINK_STATE_SIZE(TM)					\
	(ntohl(((struct lf_topo_map *)(TM))->num_links_32) 		\
	 * sizeof(lf_topo_link_state_t))

/*
 * When sending the topological map of the fabric over the wire, the
 * first thing send will be a header defining the map.  Then, the map elements
 * will be sent in order:  1 struct for each xbar, 1 struct for each NIC,
 * and then 1 struct for each link.
 */
struct lf_topo_map {
  uint32_t map_id_32;
  uint32_t num_hosts_32;	/* number of hosts in map */
  uint32_t num_xbars_32;
  uint32_t num_nics_32;
  uint32_t num_links_32;
};

/*
 * Topo node types
 */
enum lf_topo_node_type {
  LF_TOPO_NIC,
  LF_TOPO_XBAR
};

/*
 * wire representation of an xbar
 */
struct lf_topo_xbar {
  uint32_t node_id_32;
  uint32_t xbar_id_32;
  uint16_t num_ports_16;
  uint8_t clos_level_8;
  uint8_t quadrant_disable_8;
};

/* xbar node_id will is an amalgam of enclosure index, slot number, and 
 * xbar number within the linecard
 */
#define LF_TOPO_XBAR_NODE_ID(EID, SLOT, XBAR) \
		(((EID)<<16)|((SLOT)<<8)|(LF_TOPO_XBAR<<7)|(XBAR))


/*
 * Wire representation of a NIC
 */
struct lf_topo_nic {
  uint32_t node_id_32;
  uint8_t mac_addr[6];
  uint16_t num_ports_16;		/* NIC interfaces */
  uint32_t partition_32;		/* partition membership */
};

/* A NICs node_id is an amalgam of its host index and slot in host */
#define LF_TOPO_NIC_NODE_ID(HID, SLOT) (((HID)<<8)|(LF_TOPO_NIC<<7)|(SLOT))
#define LF_TOPO_NIC_HOST_ID(NI) ((NI)>>8)
#define LF_TOPO_NIC_HOST_NIC_ID(NI) ((NI)&0x7F)

/*
 * wire representation of a link between two nodes
 */
struct lf_topo_link {
  uint32_t node_index_32[2];	/* index into NIC or xbar array */
};

/*
 * State of a link - index in this array is same as index in link array.
 * It is a seperate array so that it may be sent and updated seperately from
 * the map to reduce network traffic.  There is one link_state per link.
 */
typedef uint8_t lf_topo_link_state_t;

/* definitions for composing and decomposing node_index for a link */
#define LF_TOPO_NODE_INDEX(INDEX, TYPE, PORT) (((INDEX)<<8)|((TYPE)<<7)|(PORT))
#define LF_TOPO_XBAR_INDEX(I, P) LF_TOPO_NODE_INDEX(I, LF_TOPO_XBAR, P)
#define LF_TOPO_NIC_INDEX(I, P) LF_TOPO_NODE_INDEX(I, LF_TOPO_NIC, P)

#define LF_TOPO_NI_INDEX(NI) ((NI)>>8)
#define LF_TOPO_NI_TYPE(NI) (((NI) >> 7) & 1)
#define LF_TOPO_NI_PORT(NI) ((NI)&0x7f)
#define LF_TOPO_NI_XBAR(NI) (LF_TOPO_NI_TYPE(NI) == LF_TOPO_XBAR)

#endif /* _lf_topo_map_h_ */

==> mx2g_1.1.4.tar.gz: mx-1.1.4/fma/libfma/lf_topo_map.h <==
#ifndef _lf_topo_map_h_
#define _lf_topo_map_h_

/*
 * This is a purely topological map of the network in a form
 * suitable for sending over the network
 */

/*
 * The topo map is represented in memory as one "struct lf_topo_map"
 * follow by "num_xbars" "struct lf_topo_xbar"s, etc.  The following
 * macros are for getting to each array
 */
#define LF_TOPO_XBAR_ARRAY(TM)						\
  ((struct lf_topo_xbar *)((void *)(TM) + sizeof(struct lf_topo_map)))
#define LF_TOPO_NIC_ARRAY(TM)						\
  ((struct lf_topo_nic *)((void *)LF_TOPO_XBAR_ARRAY(TM)		\
   + (sizeof(struct lf_topo_xbar) *					\
     ntohl(((struct lf_topo_map *)(TM))->num_xbars_32))))
#define LF_TOPO_LINK_ARRAY(TM)						\
  ((struct lf_topo_link *)((void *)LF_TOPO_NIC_ARRAY(TM) 		\
   + (sizeof(struct lf_topo_nic) * 					\
     ntohl(((struct lf_topo_map *)(TM))->num_nics_32))))
#define LF_TOPO_LINK_STATE_ARRAY(TM)					\
  ((lf_topo_link_state_t *)((void *)LF_TOPO_LINK_ARRAY(TM)		\
   + (sizeof(struct lf_topo_link) * 					\
     ntohl(((struct lf_topo_map *)(TM))->num_links_32))))

#define LF_TOPO_MAP_SIZE(XBARS, NICS, LINKS)				\
   (sizeof(struct lf_topo_map)						\
   + (sizeof(struct lf_topo_xbar) * (XBARS))				\
   + (sizeof(struct lf_topo_nic) * (NICS))				\
   + (sizeof(struct lf_topo_link) * (LINKS))				\
   + (sizeof(lf_topo_link_state_t) * (LINKS)))
#define LF_TOPO_LINK_STATE_SIZE(TM)					\
	(ntohl(((struct lf_topo_map *)(TM))->num_links_32) 		\
	 * sizeof(lf_topo_link_state_t))

/*
 * When sending the topological map of the fabric over the wire, the
 * first thing send will be a header defining the map.  Then, the map elements
 * will be sent in order:  1 struct for each xbar, 1 struct for each NIC,
 * and then 1 struct for each link.
 */
struct lf_topo_map {
  uint32_t map_id_32;
  uint8_t mapper_mac_addr[6];	/* MAC address of mapper generating this map */
  uint8_t pad_after_mac[2];
  uint32_t num_hosts_32;	/* number of hosts in map */
  uint32_t num_xbars_32;
  uint32_t num_nics_32;
  uint32_t num_links_32;
};

/*
 * Topo node types
 */
enum lf_topo_node_type {
  LF_TOPO_NONE=0,
  LF_TOPO_NIC=1,
  LF_TOPO_XBAR=2
};

/*
 * wire representation of an xbar
 */
struct lf_topo_xbar {
  uint32_t node_id_32;
  uint32_t xbar_id_32;
  uint16_t num_ports_16;
  uint8_t clos_level_8;
  uint8_t quadrant_disable_8;
};

/* xbar node_id is the xbar index in the fabric
 */
#define LF_TOPO_XBAR_NODE_ID(XBAR) (XBAR)

/*
 * Wire representation of a NIC
 */
struct lf_topo_nic {
  uint32_t node_id_32;
  uint8_t mac_addr[6];
  uint8_t num_ports_8;			/* NIC interfaces */
  uint8_t firmware_type_8;		/* GM, MX, 2Z, etc. */
  uint16_t partition_16;		/* partition membership */
  uint16_t mag_id_16;			/* myrinet link aggregation ID */
  uint16_t flags_16;			/* misc flags */
  uint8_t tn_pad[2];			/* pad to 4 bytes */
};

/* A NICs node_id is an amalgam of its host index and slot in host */
#define LF_TOPO_NIC_NODE_ID(HID, SLOT) (((HID)<<8)|(SLOT))
#define LF_TOPO_NIC_HOST_ID(NI) ((NI)>>8)
#define LF_TOPO_NIC_HOST_NIC_ID(NI) ((NI)&0xFF)

/*
 * wire representation of a link between two nodes
 */
struct lf_topo_link {
  uint32_t node_index_32[2];	/* index into NIC or xbar array */
  uint32_t verifier_32;		/* node encoding of NIC/port to verify */
};

/*
 * State of a link - index in this array is same as index in link array.
 * It is a seperate array so that it may be sent and updated seperately from
 * the map to reduce network traffic.  There is one link_state per link.
 */
typedef uint8_t lf_topo_link_state_t;

/* definitions for composing and decomposing node_index for a link */
#define LF_TOPO_NODE_INDEX(INDEX, TYPE, PORT) (((INDEX)<<8)|((TYPE)<<6)|(PORT))
#define LF_TOPO_VERIFIER_INDEX LF_TOPO_NODE_INDEX  /* TYPE == which end */
#define LF_TOPO_XBAR_INDEX(I, P) LF_TOPO_NODE_INDEX(I, LF_TOPO_XBAR, P)
#define LF_TOPO_NIC_INDEX(I, P) LF_TOPO_NODE_INDEX(I, LF_TOPO_NIC, P)

#define LF_TOPO_NI_INDEX(NI) ((NI)>>8)
#define LF_TOPO_NI_TYPE(NI) (((NI) >> 6) & 0x03)
#define LF_TOPO_NI_WHICH_END(NI) (((NI) >> 6) & 0x01)	/* for verifier */
#define LF_TOPO_NI_PORT(NI) ((NI)&0x3f)
#define LF_TOPO_NI_XBAR(NI) (LF_TOPO_NI_TYPE(NI) == LF_TOPO_XBAR)

/*
 * Special MAC addr for FMS-generated maps
 */
#define FMS_MAPPER_MAC "FMSMAP"

/*
 * prototypes
 */
struct lf_fabric;
struct lf_topo_map *lf_create_topo_map(struct lf_fabric *fp, int *ret_map_size);

#endif /* _lf_topo_map_h_ */
